#include "pw/scripting/script.hpp"

#include "scripting.hpp"

//#include "script_core.hpp"
//#include "script_system.hpp"
//#include "script_scene.hpp"
//#include "script_visual.hpp"

#include "pw/core/debug.hpp"

namespace pw {

struct script::state {

	sol::state _state;
	sol::table _namespace;

	void load_modules();
	void load(const std::string& name);

	state() {

		// create global pw namespace
		_namespace = _state.create_named_table("pw");

		// add the script as a user type
		_namespace.new_usertype<state>("script",
									   "initialize",&state::load_modules,
									   "load",&state::load
									   );

		// add this instance as "script"
		_namespace.set("script",this);
	}

	int run(const std::string &s);

};

int script::state::run(const std::string &s)
{
	return _state.script(s.c_str()).get<int>();
}

void script::state::load(const std::string& name)
{
	lua_runtime::register_function_list[name](_state,_namespace);
}

void script::state::load_modules() {

	// open all libraries
	_state.open_libraries();

	for (auto lib_reg : lua_runtime::register_function_list) {
		lib_reg.second(_state,_namespace);
		debug::d() << lib_reg.first;
	}

//	script_core::load(_namespace);
//	script_system::load(_namespace);
//	script_scene::load(_namespace);
//    script_visual::load(_namespace);

}

//
//
//

script::script()
	: _state(make_unique<script::state>())
{
}

script::~script()
{
	_state = nullptr;
}

int script::run(const std::string &s)
{
	return _state->run(s);

}


}




